from __future__ import annotations

import datetime
import logging
from typing import TYPE_CHECKING, Any

from dateutil.parser import parse as date_parse
from django.db import models
from django.utils.translation import gettext_lazy as _

from ..api import get
from ..signals import on_completion, on_status_change

if TYPE_CHECKING:
    from .event import Event

logger = logging.getLogger(__name__)


class BaseModel(models.Model):
    """Base model used to set timestamps."""

    # used to format the href - override in subclasses
    base_href = ""

    amiqus_id = models.CharField(
        "Amiqus ID",
        unique=True,
        max_length=40,
        help_text=_("The id returned from the Amiqus API."),
    )
    created_at = models.DateTimeField(
        help_text=_("The timestamp returned from the Amiqus API."),
        blank=True,
        null=True,
    )
    raw = models.JSONField(
        help_text=_("The raw JSON returned from the API."), blank=True, null=True
    )

    class Meta:
        abstract = True

    @property
    def href(self) -> str:
        """Return the href from base_href."""
        return f"{self.base_href}/{self.amiqus_id}"

    def save(self, *args: Any, **kwargs: Any) -> BaseModel:
        """Save object and return self (for chaining methods)."""
        self.full_clean()
        super().save(*args, **kwargs)
        return self

    def parse(self, raw_json: dict) -> BaseModel:
        """Parse the raw value out into other properties."""
        self.raw = raw_json
        self.amiqus_id = self.raw["id"]
        self.status = self.raw["status"]
        self.created_at = date_parse(self.raw["created_at"])

        return self

    def fetch(self) -> BaseModel:
        """
        Fetch the object JSON from the remote API.

        Named after the git operation - this will call the API for the
        latest JSON representation, and update the local fields, but will
        not save the updates. This is useful for inspecting the API response
        without making permanent changes to the object. It can also be used
        to interact with the API without saving an objects:

        >>> obj = Record(amiqus_id='123').fetch()

        Returns the updated object (unsaved).

        """
        return self.parse(get(self.href))

    def pull(self) -> BaseModel:
        """
        Update the object from the remote API.

        Named after the git operation - this will call fetch(), and
        then save the object.

        Returns the updated object (saved).

        """
        return self.fetch().save()


class BaseQuerySet(models.QuerySet):
    """Custom queryset for models subclassing BaseModel."""

    def fetch(self) -> None:
        """Call fetch method on all objects in the queryset."""
        for obj in self:
            try:
                obj.fetch()
            except Exception:  # noqa: B902
                logger.exception("Failed to fetch Amiqus object: %r", obj)

    def pull(self) -> None:
        """Call pull method on all objects in the queryset."""
        for obj in self:
            try:
                obj.pull()
            except Exception:  # noqa: B902
                logger.exception("Failed to pull Amiqus object: %r", obj)


class BaseStatusModel(BaseModel):
    """Base class for models with a status field."""

    class Status(models.TextChoices):
        """Combined list of states values for record / check."""

        PENDING = ("pending", _("Pending"))
        SUBMITTED = ("submitted", _("Submitted"))
        ACCEPTED = ("accepted", _("Accepted"))
        REJECTED = ("rejected", _("Cancelled"))
        REFER = ("refer", _("Refer"))
        PAUSED = ("paused", _("Paused"))

    status = models.CharField(
        max_length=20,
        help_text=_("The current status of the record / check (from API)."),
        choices=Status.choices,
        db_index=True,
        blank=True,
        null=True,
    )
    updated_at = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_("The timestamp of the most recent status change (from API)."),
    )

    class Meta:
        abstract = True

    def events(self) -> BaseQuerySet:
        """Return queryset of Events related to this object."""
        # prevents circ. import
        from .event import Event

        return Event.objects.filter(
            amiqus_id=self.amiqus_id, resource_type=self._meta.model_name
        )

    def update_status(self, event: Event) -> BaseStatusModel:
        """
        Update the status field of the object and fire signal(s).

        When the app receives an event callback from Amiqus, we update
        the status of the relevant Record/Check, and then fire the
        signals that allow external apps to hook in to this event.

        If the update is a change to 'complete', then we fire a second
        signal - 'complete' is the terminal state change, and therefore
        of most interest to clients - typically the on_status_update
        signal would be registered for logging a complete history of
        changes, whereas the on_completion signal would be used to do
        something more useful - updating the status of the user, sending
        them an email etc.

        Args:
            event: Event object containing the update information

        Returns the updated object.

        """
        # we're doing a lot of marshalling from JSON to python, so this assert
        # just ensures we do actually have a datetime at this point
        if not isinstance(event.completed_at, datetime.datetime):
            raise ValueError("event.completed_at is not a datetime object")
        # swap statuses around so we record old / new
        self.status, old_status = event.status, self.status
        self.updated_at = event.completed_at
        try:
            self.pull()
        except Exception:  # noqa: B902
            # even if we can't get latest, we should save the changes we
            # have already made to the object
            logger.warning("Unable to pull latest from Amiqus: '%r'", self)
            self.save()
        on_status_change.send(
            self.__class__,
            instance=self,
            event=event.action,
            status_before=old_status,
            status_after=event.status,
        )
        if event.status == self.Status.ACCEPTED.value:  # type: ignore[attr-defined]
            on_completion.send(self.__class__, instance=self)
        return self

    def parse(self, raw_json: dict) -> BaseStatusModel:
        """Parse the raw value out into other properties."""
        super().parse(raw_json)
        self.status = self.raw["status"]
        return self

    @property
    def is_clear(self) -> bool:
        """Return True/False to whether a record is successful and clear."""
        return self.status == self.Status.ACCEPTED.value  # type: ignore[attr-defined]



from django.apps import AppConfig


class AmiqusAppConfig(AppConfig):
    """AppConfig for Django-Amiqus."""

    name = "amiqus"
    verbose_name = "Amiqus"
    default_auto_field = "django.db.models.AutoField"



"""
Amiqus webhook handler.

The Amiqus API will send a webhook to the configured URL when an event
occurs, which we then use to update the relevant object.

"""
from __future__ import annotations

import json
import logging

from django.http import HttpRequest, HttpResponse
from django.utils.timezone import now
from django.views.decorators.csrf import csrf_exempt

from .decorators import verify_signature
from .models import Client, Event, Record
from .settings import LOG_EVENTS

logger = logging.getLogger(__name__)


@csrf_exempt
@verify_signature()
def status_update(request: HttpRequest) -> HttpResponse:  # noqa: C901
    """
    Handle event callbacks from the API.

    This is the request handler which does little other than log the
    request and call the _webhook function which does the processing.
    Done like this to make it easier to test without having to set up
    request objects.

    NB This view function will always return a 200 status - in order to
    prevent Amiqus from endlessly retrying. The only exceptions to this
    are caused by the verify_signature decorator - if it cannot verify
    the callback, then it will return a 403 - which should be ok, as if
    Amiqus sends the request it should never fail...

    """
    received_at = now()
    logger.debug("Received Amiqus callback: %s", request.body)
    if not (data := json.loads(request.body)):
        logger.exception("Received empty Amiqus webhook body.")
        return HttpResponse("Empty webhook.")
    try:
        trigger = data["trigger"]
        alias = trigger["alias"]
    except KeyError:
        logger.exception("Invalid Amiqus webhook body.")
        return HttpResponse("Invalid webhook body.")

    entity_type = alias.split(".")[0]
    if entity_type not in ("client", "record"):
        logger.error("Invalid entity_type: %s", entity_type)
        return HttpResponse("Invalid event trigger.")
    event = Event(received_at=received_at)

    try:
        resource = event.parse(data, entity_type=entity_type).resource
        resource.update_status(event)
        if LOG_EVENTS:
            event.save()
        return HttpResponse("Update processed.")
    except KeyError as ex:
        logger.exception("Missing Amiqus event content.", exc_info=ex)
        return HttpResponse("Unexpected event content.")
    except ValueError:
        logger.exception("Unknown Amiqus resource type: %s", event.resource_type)
        return HttpResponse("Unknown resource type.")
    except Record.DoesNotExist:
        # TODO(source-of-truth) Create Record when *new* request is spawned from Amiqus
        # Missing Records can be synchronised as we utilise a reference shared
        # between the user, and Amiqus itself.
        logger.exception("Amiqus record does not exist: %s", event.amiqus_id)
        # 1. Get Record from API.
        # 2. Get Client attached to Record
        # 3. Use their reference to match with a user on system
        # 4. Create record as normal
        return HttpResponse("Record not found.")
    except Client.DoesNotExist:
        logger.warning("Amiqus client does not exist: %s", event.amiqus_id)
        return HttpResponse("Client not found.")
    except Exception:  # noqa: B902
        logger.exception("Amiqus update could not be processed.")
        return HttpResponse("Unknown error.")



from __future__ import annotations

import logging

from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from ..settings import scrub_check_data
from .base import BaseQuerySet, BaseStatusModel
from .record import Record

logger = logging.getLogger(__name__)


class CheckQuerySet(BaseQuerySet):
    """Check model queryset."""

    def create_check(self, record: Record, raw: dict) -> Check:
        """Create a new Check from the raw JSON."""
        logger.debug("Creating new Onfido check from JSON: %s", raw)
        return Check(user=record.user, amiqus_record=record).parse(raw).save()


class Check(BaseStatusModel):
    """Specific checks associated with a Record."""

    class CheckStatus(models.TextChoices):
        """Combined list of states values for record / check."""

        PENDING = ("pending", _("Pending"))
        SUBMITTED = ("submitted", _("Submitted"))
        ACCEPTED = ("accepted", _("Accepted"))
        REJECTED = ("rejected", _("Cancelled"))
        REFER = ("refer", _("Refer"))
        FAILED = ("failed", _("Failed"))
        PAUSED = ("paused", _("Paused"))

    base_href = "checks"

    class CheckType(models.TextChoices):
        # https://documentation.onfido.com/#check-names-in-api
        DOCUMENT = ("document", "Document")
        DOCUMENT_WITH_ADDRESS_INFORMATION = (
            "document_with_address_information",
            "Document with Address Information",
        )
        DOCUMENT_WITH_DRIVING_LICENCE_INFORMATION = (
            "document_with_driving_licence_information",
            "Document with Driving Licence Information",
        )
        FACIAL_SIMILARITY_PHOTO = (
            "facial_similarity_photo",
            "Facial Similarity (photo)",
        )
        FACIAL_SIMILARITY_PHOTO_FULLY_AUTO = (
            "facial_similarity_photo_fully_auto",
            "Facial Similarity (auto)",
        )
        FACIAL_SIMILARITY_VIDEO = (
            "facial_similarity_video",
            "Facial Similarity (video)",
        )
        KNOWN_FACES = ("known_faces", "Known Faces")
        IDENTITY_ENHANCED = ("identity_enhanced", "Identity (enhanced)")
        WATCHLIST_ENHANCED = ("watchlist_enhanced", "Watchlist (enhanced)")
        WATCHLIST_STANDARD = ("watchlist_standard", "Watchlist")
        WATCHLIST_PEPS_ONLY = ("watchlist_peps_only", "Watchlist (PEPs only)")
        WATCHLIST_SANCTIONS_ONLY = (
            "watchlist_sanctions_only",
            "Watchlist (sanctions only)",
        )
        PROOF_OF_ADDRESS = ("proof_of_address", "Proof of Address")
        RIGHT_TO_WORK = ("right_to_work", "Right to Work")

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        help_text=_(
            "The Django user (denormalised from Client to make navigation easier)."
        ),  # noqa
        related_name="amiqus_checks",
    )
    amiqus_record = models.ForeignKey(
        Record,
        on_delete=models.CASCADE,
        help_text=_("Record to which this check is attached."),
        related_name="checks",
    )
    check_type = models.CharField(
        max_length=50,
        choices=CheckType.choices,
        help_text=_(
            "The name of the check - see https://documentation.onfido.com/#checks"
        ),
    )

    objects = CheckQuerySet.as_manager()

    def __str__(self) -> str:
        return "{} for {}".format(self.get_check_type_display().capitalize(), self.user)

    def __repr__(self) -> str:
        return "<Check id={} type='{}' user_id={}>".format(
            self.id, self.check_type, self.user.id
        )

    def parse(self, raw_json: dict) -> Check:
        """
        Parse the raw value out into other properties.

        Before parsing the data, this method will call the
        scrub_check_data function to remove sensitive data
        so that it is not saved into the local object.

        """
        super().parse(scrub_check_data(raw_json))
        self.check_type = self.raw["type"]
        self.status = self.raw["status"]
        # Once the V2 API has been implemented, this won't be necessary.
        if isinstance(self.status, int):
            self.status = self.deprecated_status_mapper(self.status)
        return self

    def deprecated_status_mapper(self, status: int) -> CheckStatus | None:
        """
        Return a v2 status for a v1 endpoint.

        V1 endpoint returns an integer based status value.
        V2 endpoint returns and uses a text based status value.
        This function patches the two.
        """
        return {
            0: self.CheckStatus.PENDING.value,  # type: ignore[attr-defined]
            1: self.CheckStatus.SUBMITTED.value,  # type: ignore[attr-defined]
            2: self.CheckStatus.ACCEPTED.value,  # type: ignore[attr-defined]
            3: self.CheckStatus.REJECTED.value,  # type: ignore[attr-defined]
            4: self.CheckStatus.REFER.value,  # type: ignore[attr-defined]
            5: self.CheckStatus.FAILED.value,  # type: ignore[attr-defined]
            6: self.CheckStatus.PAUSED.value,  # type: ignore[attr-defined]
        }[status]



from __future__ import annotations

import datetime
import logging
from typing import TYPE_CHECKING

from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from ..settings import scrub_client_data
from ..signals import on_completion, on_status_change
from .base import BaseModel, BaseQuerySet

if TYPE_CHECKING:
    from .event import Event

logger = logging.getLogger(__name__)


class ClientManager(models.Manager):
    def create_client(self, user: settings.AUTH_USER_MODEL, raw: dict) -> Client:
        """Create a new client in Amiqus from a user."""
        logger.debug("Creating new Amiqus client from JSON: %s", raw)
        client = Client(user=user)
        client.parse(raw)
        client.save()
        return client


class Client(BaseModel):
    """An Amiqus client record."""

    base_href = "clients"

    class ClientStatus(models.TextChoices):
        UNKNOWN = ("", _("Unknown"))
        APPROVED = ("approved", _("Approved"))
        REJECTED = ("rejected", _("Rejected"))
        FOR_REVIEW = ("pending", _("Needs review"))

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        help_text=_("Django user that maps to this client."),
        related_name="amiqus_clients",
    )

    status = models.CharField(
        max_length=20,
        help_text=_("The current status of the Client (from API)."),
        choices=ClientStatus.choices,
        db_index=True,
        blank=True,
        null=True,
    )

    objects = ClientManager.from_queryset(BaseQuerySet)()

    def __str__(self) -> str:
        return str(self.user)

    def __repr__(self) -> str:
        return "<Client id={} user_id={}>".format(self.id, self.user.id)

    def parse(self, raw_json: dict) -> Client:
        """
        Parse the raw value out into other properties.

        Before parsing the data, this method will call the
        scrub_check_data function to remove sensitive data
        so that it is not saved into the local object.

        """
        super().parse(scrub_client_data(raw_json))
        return self

    def update_status(self, event: Event) -> Client:
        """
        Update the status field of a Client and fire signal(s).

        When the app receives an event callback from Amiqus, we update
        the status of the Client, and then fire the signals that allow
        external apps to hook in to this event.

        If the update is a change to 'approve', then we fire a second
        signal - 'approved' is the terminal state change, and therefore
        of most interest to clients - typically the on_status_update
        signal would be registered for logging a complete history of
        changes, whereas the on_completion signal would be used to do
        something more useful - updating the status of the user, sending
        them an email etc.

        Args:
            event: Event object containing the update information

        Returns the updated object.

        """
        # we're doing a lot of marshalling from JSON to python, so this assert
        # just ensures we do actually have a datetime at this point
        if not isinstance(event.completed_at, datetime.datetime):
            raise ValueError("event.completed_at is not a datetime object")
        # swap statuses around so we record old / new
        self.status, old_status = event.status, self.status
        self.updated_at = event.completed_at
        try:
            self.pull()
        except Exception as e:  # noqa: B902
            # even if we can't get latest, we should save the changes we
            # have already made to the object
            logger.warning("Unable to pull latest from Amiqus: '%r'", self)
            self.save()
            raise e
        on_status_change.send(
            self.__class__,
            instance=self,
            event=event.action,
            status_before=old_status,
            status_after=event.status,
        )
        if event.status == self.ClientStatus.APPROVED.value:  # type: ignore[attr-defined]
            on_completion.send(self.__class__, instance=self)
        return self



from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from dateutil.parser import parse as date_parse
from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from .base import BaseQuerySet, BaseStatusModel

if TYPE_CHECKING:
    from .client import Client

logger = logging.getLogger(__name__)


class RecordQuerySet(BaseQuerySet):
    """Record model manager."""

    def create_record(self, client: Client, raw: dict) -> Record:
        """Create a new Record object from the raw JSON."""
        logger.debug("Creating new Amiqus record from JSON: %s", raw)
        return Record(user=client.user, client=client).parse(raw).save()


class Record(BaseStatusModel):
    """The state of an individual record made against an Client."""

    base_href = "records"

    class RecordStatus(models.TextChoices):
        PENDING = ("pending", _("Pending"))
        STARTED = ("started", _("Started"))
        COMPLETE = ("complete", _("Complete"))
        INCOMPLETE = ("incomplete", _("Incomplete"))
        WAITING = ("waiting", _("Waiting"))
        FAILED = ("failed", _("Failed"))
        EMPTY = ("empty", _("Empty"))
        AMENDMENTS = ("amendments", _("Amendments"))
        PAUSED = ("paused", _("Paused"))
        REVIEWED = ("reviewed", _("Reviewed"))
        UNKNOWN = ("unknown", _("Unknown"))

    class RecordType(models.TextChoices):
        CREDIT = "credit"
        CRIMINAL_RECORD = "criminal_record"
        DOCUMENT = "document"
        DUMMY = "dummy"
        IDENTITY = "identity"
        THORNTONS_ONBOARDING = "thorntons_onboarding"
        WATCHLIST = "watchlist"

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        help_text=_(
            "The Django user (denormalised from Client to make navigation easier)."
        ),  # noqa
        related_name="amiqus_records",
    )

    perform_url = models.CharField(max_length=200, blank=True, null=True)

    status = models.CharField(
        max_length=20,
        help_text=_("The current status of the record / check (from API)."),
        choices=RecordStatus.choices,
        db_index=True,
        blank=True,
        null=True,
    )

    client = models.ForeignKey(
        "amiqus.Client",
        on_delete=models.CASCADE,
        help_text=_("The client for whom the record is being made."),
        related_name="records",
    )

    objects = RecordQuerySet.as_manager()

    def __str__(self) -> str:
        return f"Amiqus record for {self.user}"

    def __repr__(self) -> str:
        return (
            f"<Record amiqus_id={self.amiqus_id} id={self.id} user_id={self.user_id}>"
        )

    def parse(self, raw_json: dict) -> Record:
        """Parse the raw value out into other properties."""
        self.raw = raw_json
        self.amiqus_id = self.raw["id"]
        self.status = self.raw["status"]
        if isinstance(self.status, int):
            self.status = self.deprecated_status_mapper(self.status)
        self.created_at = date_parse(self.raw["created_at"])
        try:
            self.perform_url = self.raw["links"]["perform"]
        except KeyError:
            # There's no url to perform as part of this request.
            pass
        try:
            from .check import Check

            # Loop through all checks in the response, and update our downstream models.
            for response_check in self.raw["checks"]["data"]:
                check, _ = Check.objects.get_or_create(
                    amiqus_id=response_check["id"],
                    user=self.user,
                    check_type=response_check["type"],
                    amiqus_record=self,
                )
                check.parse(response_check)
                # Strictly speaking we shouldn't do this here.
                # There's currently no API endpoint to fire a sequential update.
                # So we pick the fields from the record fetch.
                check.save()
        except KeyError:
            # We have no checks
            pass
        return self

    def deprecated_status_mapper(self, status: int) -> RecordStatus | None:
        """
        Return a v2 status for a v1 endpoint.

        V1 endpoint returns an integer based status value. V2 endpoint
        returns and uses a text based status value. This function
        patches the two.

        """
        CHECK_STATUS_MAP = {
            0: self.RecordStatus.PENDING.value,  # type: ignore[attr-defined]
            1: self.RecordStatus.STARTED.value,  # type: ignore[attr-defined]
            2: self.RecordStatus.UNKNOWN.value,  # type: ignore[attr-defined]
            3: self.RecordStatus.UNKNOWN.value,  # type: ignore[attr-defined]
            4: self.RecordStatus.UNKNOWN.value,  # type: ignore[attr-defined]
            5: self.RecordStatus.FAILED.value,  # type: ignore[attr-defined]
            6: self.RecordStatus.UNKNOWN.value,  # type: ignore[attr-defined]
            7: self.RecordStatus.UNKNOWN.value,  # type: ignore[attr-defined]
            8: self.RecordStatus.REVIEWED.value,  # type: ignore[attr-defined]
        }
        return CHECK_STATUS_MAP[status]



# Generated by Django 4.1.7 on 2023-03-30 12:38

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Client",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "amiqus_id",
                    models.CharField(
                        help_text="The id returned from the Amiqus API.",
                        max_length=40,
                        unique=True,
                        verbose_name="Amiqus ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        blank=True,
                        help_text="The timestamp returned from the Amiqus API.",
                        null=True,
                    ),
                ),
                (
                    "raw",
                    models.JSONField(
                        blank=True,
                        help_text="The raw JSON returned from the API.",
                        null=True,
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("None", "Unknown"),
                            ("approved", "Approved"),
                            ("rejected", "Rejected"),
                            ("pending", "Needs review"),
                        ],
                        db_index=True,
                        help_text="The current status of the Client (from API).",
                        max_length=20,
                        null=True,
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        help_text="Django user that maps to this client.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="amiqus_clients",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Event",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "amiqus_id",
                    models.CharField(
                        help_text="The Amiqus ID of the related resource.",
                        max_length=40,
                        verbose_name="Amiqus ID",
                    ),
                ),
                (
                    "resource_type",
                    models.CharField(
                        help_text="The resource_type returned from the API callback.",
                        max_length=20,
                    ),
                ),
                (
                    "action",
                    models.CharField(
                        help_text="The event name as returned from the API callback.",
                        max_length=20,
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        help_text="The status of the object after the event.",
                        max_length=20,
                    ),
                ),
                (
                    "completed_at",
                    models.DateTimeField(
                        blank=True,
                        help_text="The timestamp returned from the Amiqus API.",
                        null=True,
                    ),
                ),
                (
                    "received_at",
                    models.DateTimeField(
                        help_text="The timestamp when the server received the event."
                    ),
                ),
                (
                    "raw",
                    models.JSONField(
                        blank=True,
                        help_text="The raw JSON returned from the API.",
                        null=True,
                    ),
                ),
            ],
            options={
                "ordering": ["completed_at"],
            },
        ),
        migrations.CreateModel(
            name="Record",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "amiqus_id",
                    models.CharField(
                        help_text="The id returned from the Amiqus API.",
                        max_length=40,
                        unique=True,
                        verbose_name="Amiqus ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        blank=True,
                        help_text="The timestamp returned from the Amiqus API.",
                        null=True,
                    ),
                ),
                (
                    "raw",
                    models.JSONField(
                        blank=True,
                        help_text="The raw JSON returned from the API.",
                        null=True,
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        blank=True,
                        help_text="The timestamp of the most recent status change (from API).",
                        null=True,
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("pending", "Pending"),
                            ("started", "Started"),
                            ("complete", "Complete"),
                            ("incomplete", "Incomplete"),
                            ("waiting", "Waiting"),
                            ("failed", "Failed"),
                            ("empty", "Empty"),
                            ("amendments", "Amendments"),
                            ("paused", "Paused"),
                            ("reviewed", "Reviewed"),
                        ],
                        db_index=True,
                        help_text="The current status of the record / check (from API).",
                        max_length=20,
                        null=True,
                    ),
                ),
                (
                    "client",
                    models.ForeignKey(
                        help_text="The client for whom the record is being made.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="records",
                        to="amiqus.client",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        help_text="The Django user (denormalised from Client to make navigation easier).",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="amiqus_records",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Check",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "amiqus_id",
                    models.CharField(
                        help_text="The id returned from the Amiqus API.",
                        max_length=40,
                        unique=True,
                        verbose_name="Amiqus ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        blank=True,
                        help_text="The timestamp returned from the Amiqus API.",
                        null=True,
                    ),
                ),
                (
                    "raw",
                    models.JSONField(
                        blank=True,
                        help_text="The raw JSON returned from the API.",
                        null=True,
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("pending", "Pending"),
                            ("submitted", "Submitted"),
                            ("accepted", "Accepted"),
                            ("rejected", "Cancelled"),
                            ("refer", "Refer"),
                            ("paused", "Paused"),
                        ],
                        db_index=True,
                        help_text="The current status of the record / check (from API).",
                        max_length=20,
                        null=True,
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        blank=True,
                        help_text="The timestamp of the most recent status change (from API).",
                        null=True,
                    ),
                ),
                (
                    "check_type",
                    models.CharField(
                        choices=[
                            ("document", "Document"),
                            (
                                "document_with_address_information",
                                "Document with Address Information",
                            ),
                            (
                                "document_with_driving_licence_information",
                                "Document with Driving Licence Information",
                            ),
                            ("facial_similarity_photo", "Facial Similarity (photo)"),
                            (
                                "facial_similarity_photo_fully_auto",
                                "Facial Similarity (auto)",
                            ),
                            ("facial_similarity_video", "Facial Similarity (video)"),
                            ("known_faces", "Known Faces"),
                            ("identity_enhanced", "Identity (enhanced)"),
                            ("watchlist_enhanced", "Watchlist (enhanced)"),
                            ("watchlist_standard", "Watchlist"),
                            ("watchlist_peps_only", "Watchlist (PEPs only)"),
                            ("watchlist_sanctions_only", "Watchlist (sanctions only)"),
                            ("proof_of_address", "Proof of Address"),
                            ("right_to_work", "Right to Work"),
                        ],
                        help_text="The name of the check - see https://documentation.onfido.com/#checks",
                        max_length=50,
                    ),
                ),
                (
                    "amiqus_record",
                    models.ForeignKey(
                        help_text="Record to which this check is attached.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="checks",
                        to="amiqus.record",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        help_text="The Django user (denormalised from Client to make navigation easier).",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="amiqus_checks",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



"""
Amiqus signals.

Signals are fired when corresponding callbacks are received.

See https://documentation.onfido.com/#webhooks

"""
from django.dispatch import Signal

# fired after the status of a check /record is updated
# e.g. (obj, 'check.completed', 'pending', 'complete')
# providing_args=["instance", "event", "status_before", "status_after"]
on_status_change = Signal()

# 'complete' is the terminal state of both records and checks, and as
# such it gets special treatment - it's the event that most people will
# be interested, so instead of having to filter out the on_status_change
# signal that results in completion, we have a dedicated signal.
# providing_args=["instance", "completed_at"]
on_completion = Signal()



# Generated by Django 4.1.7 on 2023-03-31 12:43

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("amiqus", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="record",
            name="perform_url",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="record",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("pending", "Pending"),
                    ("started", "Started"),
                    ("complete", "Complete"),
                    ("incomplete", "Incomplete"),
                    ("waiting", "Waiting"),
                    ("failed", "Failed"),
                    ("empty", "Empty"),
                    ("amendments", "Amendments"),
                    ("paused", "Paused"),
                    ("reviewed", "Reviewed"),
                    ("unknown", "Unknown"),
                ],
                db_index=True,
                help_text="The current status of the record / check (from API).",
                max_length=20,
                null=True,
            ),
        ),
    ]



"""
Basic wire operations with the API - GET/POST/PUT.

This is a simple wrapper around requests.

"""
from __future__ import annotations

import logging
from urllib import parse as urlparse

import requests
from django.http import HttpResponse

from .settings import API_KEY, DEFAULT_REQUESTS_TIMEOUT

logger = logging.getLogger(__name__)

# the API HTTP root url
API_ROOT = "https://id.amiqus.co/api/"
API_ROOT_V2 = "https://id.amiqus.co/api/v2/"

API_ROOT_MAP = {
    1: API_ROOT,
    2: API_ROOT_V2,
}


class ApiError(Exception):
    """Error raised when interacting with the API."""

    def __init__(self, response: HttpResponse) -> None:
        """Initialise error from response object."""
        data = response.json()
        logger.debug("Amiqus API error: %s", data)
        super().__init__(data["error"])
        self.status_code = response.status_code
        self.error_type = data["error"]


def _url(path: str, version: int) -> str:
    """Format absolute API URL."""
    api_root = API_ROOT_MAP.get(version, API_ROOT_V2)
    return urlparse.urljoin(api_root, path)


def _headers(api_key: str = API_KEY) -> dict[str, str]:
    """Format request headers."""
    return {
        "Authorization": "Bearer {}".format(api_key),
        "Content-Type": "application/json",
    }


def _respond(response: HttpResponse) -> dict:
    """Process common response object."""
    if not str(response.status_code).startswith("2"):
        raise ApiError(response)
    data = response.json()
    logger.debug("Amiqus API response: %s", data)
    return data


def get(href: str, *, version: int = 2) -> dict:
    """Make a GET request and return the response as JSON."""
    logger.debug("Amiqus API GET request: %s", href)
    return _respond(
        requests.get(
            _url(href, version), headers=_headers(), timeout=DEFAULT_REQUESTS_TIMEOUT
        )
    )


def post(href: str, *, data: dict, version: int = 2) -> dict:
    """Make a POST request and return the response as JSON."""
    logger.debug("Amiqus API POST request: %s", href)
    return _respond(
        requests.post(
            _url(href, version),
            headers=_headers(),
            json=data,
            timeout=DEFAULT_REQUESTS_TIMEOUT,
        )
    )



# Generated by Django 4.2.7 on 2023-11-15 12:04

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("amiqus", "0002_record_perform_url_alter_record_status"),
    ]

    operations = [
        migrations.AlterField(
            model_name="client",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("", "Unknown"),
                    ("approved", "Approved"),
                    ("rejected", "Rejected"),
                    ("pending", "Needs review"),
                ],
                db_index=True,
                help_text="The current status of the Client (from API).",
                max_length=20,
                null=True,
            ),
        ),
    ]



from __future__ import annotations

import logging
from typing import TypeAlias

from dateutil.parser import parse as date_parse
from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from .client import Client
from .record import Record

UserType: TypeAlias = settings.AUTH_USER_MODEL

logger = logging.getLogger(__name__)


class Event(models.Model):
    """Used to record callback events received from the API."""

    # used to map the event name to the model it affects
    EVENT_ACTION_MAP = {
        "record.bounced": Record,
        "record.created": Record,
        "record.updated": Record,
        "record.finished": Record,
        "record.reviewed": Record,
        "client.created": Client,
        "client.record": Client,
        "client.updated": Client,
        "client.status": Client,
    }

    amiqus_id = models.CharField(
        "Amiqus ID",
        max_length=40,
        help_text=_("The Amiqus ID of the related resource."),
    )
    resource_type = models.CharField(
        max_length=20, help_text=_("The resource_type returned from the API callback.")
    )
    action = models.CharField(
        max_length=20, help_text=_("The event name as returned from the API callback.")
    )
    status = models.CharField(
        max_length=20, help_text=_("The status of the object after the event.")
    )
    completed_at = models.DateTimeField(
        help_text=_("The timestamp returned from the Amiqus API."),
        blank=True,
        null=True,
    )
    received_at = models.DateTimeField(
        help_text=_("The timestamp when the server received the event."),
    )
    raw = models.JSONField(
        help_text=_("The raw JSON returned from the API."), blank=True, null=True
    )

    class Meta:
        ordering = ["completed_at"]

    def __str__(self) -> str:
        return "{} event occurred on {}.{}".format(
            self.action, self.resource_type, self.amiqus_id
        )

    def __repr__(self) -> str:
        return "<Event id={} action='{}' amiqus_id='{}.{}'>".format(
            self.id, self.action, self.resource_type, self.amiqus_id
        )

    def _resource_manager(self) -> models.Manager:
        """Return the appropriate model manager for the resource_type."""
        try:
            return self.EVENT_ACTION_MAP[self.action].objects
        except KeyError:
            raise ValueError(f"Unknown resource action: {self.action}")

    @property
    def resource(self) -> Record | Client:
        """Return the underlying Record or Check resource."""
        return self._resource_manager().get(amiqus_id=self.amiqus_id)

    @property
    def user(self) -> UserType:
        """Return the user to whom the resource refers."""
        return self.resource.user

    def parse(self, raw_json: dict, *, entity_type: str) -> Event:
        """Parse the raw value out into other properties."""
        self.raw = raw_json
        data = self.raw["data"]
        trigger = self.raw["trigger"]
        self.resource_type = entity_type
        self.action = trigger["alias"]
        obj = data[entity_type]
        self.amiqus_id = obj["id"]
        self.completed_at = date_parse(trigger["triggered_at"])
        return self



from __future__ import annotations

from typing import Any, Iterable

from django.conf import settings

from .api import post
from .models import Check, Client, Record


def create_client(user: settings.AUTH_USER_MODEL, **kwargs: Any) -> Client:
    """
    Create an client in the Amiqus system.

    Args:
        user: a Django User instance to register as a Client.
    Kwargs:
       any kwargs passed in are merged into the data dict sent to the
       API. This enables support for additional client properties - e.g
       dob, gender, country, and any others that may change over time.

    """
    data = {
        "name": {
            "first_name": user.first_name,
            "last_name": user.last_name,
        },
        "email": user.email,
    }
    data.update(kwargs)
    response = post("clients", data=data)
    return Client.objects.create_client(user, response)


def create_record(client: Client, check_names: Iterable, **kwargs: Any) -> Record:
    """
    Create a new Record (and child Checks).

    Args:
        client: Client for whom the records are being made. check_names:
        list of strings, each of which is a valid check type.

    Kwargs:
        any kwargs passed in are merged into the data dict sent to the
        API. This enables support for additional record properties -
        e.g. redirect_uri, tags, suppress_form_emails and any other that
        may change over time. See

    Returns a new Record object, and creates the child Check objects.

    """
    data = {
        "client_id": client.amiqus_id,
        "checks": check_names,
        "reset_client_status": True,
        "send_email": True,
        "send_reminder": True,
    }
    # merge in the additional kwargs
    data.update(kwargs)

    # V2 here, this isn't yet implemented. It will error with a 405 method not allowed.
    # response = post("records", data=data)
    # We need to use API v1 for this until they add this method to v2 :) :( :)
    response = post("records", data=data, version=1)
    record = Record.objects.create_record(client=client, raw=response)

    try:
        for response_check in data["checks"]["data"]:
            Check.objects.create_check(record=record, raw=response_check)
    except KeyError:
        # No checks
        pass
    return record



from __future__ import annotations

import hashlib
import hmac
import logging
from base64 import b64encode
from functools import wraps
from typing import Any, Callable

from django.core.exceptions import ImproperlyConfigured
from django.http import HttpRequest, HttpResponse, HttpResponseForbidden

from .settings import TEST_MODE, WEBHOOK_SECURITY_TOKEN

logger = logging.getLogger(__name__)


def _hmac(token: bytes, text: bytes) -> bytes:
    """
    Calculate SHA1 HMAC digest from request body and token.

    Args:
        token: bytes, the webhook token from Amiqus API settings.
        text: string, the text to hash.

    Return the SHA1 HMAC as a string.

    """
    # This simply doesn't work. TEST_MODE should remain on until a future date.
    auth_code = hmac.new(token, text, hashlib.sha256).hexdigest()
    base64_hmac = b64encode(auth_code.encode("utf-8"))
    logger.debug("Amiqus callback request HMAC: %s", base64_hmac)
    return base64_hmac


def _match(token: bytes, request: HttpRequest) -> bool:
    """
    Calculate signature and return True if it matches header.

    Args:
        token: string, the webhook_token.
        request: an HttpRequest object from which the body content and
            X-Signature header will be extracted and matched.

    Returns True if there is a match.

    """
    try:
        signature = request.headers["X_AQID_SIGNATURE"]
        logger.debug("Amiqus callback X-Signature: %s", signature)
        logger.debug("Amiqus callback request body: %s", request.body)

        return _hmac(token, request.body) == signature
    except KeyError:
        logger.warning(
            "Amiqus callback missing X-Signature - this may be an unauthorised request."
        )
        return False
    except Exception:  # noqa: B902
        logger.exception("Error attempting to decode Amiqus signature.")
        return False


def verify_signature() -> Callable:
    """
    View function decorator used to verify Amiqus webhook signatures.

    This function uses the WEBHOOK_SECURITY_TOKEN specified in settings to calculate
    the HMAC. If it doesn't exist, then this decorator will immediately fail
    hard with an ImproperlyConfigured exception.

    If TEST_MODE is on (AMIQUS_TEST_MODE setting) then the verification will
    be ignored.

    See: https://amiqus.github.io/developers/api#tag/Webhooks

    Each webhook URL is associated with a secret token. This is
    visible in the amiqus dashboard for a given webhook.

    Events sent to your application will be signed using this token:
    verifying the request signature on your server prevents attackers
    from imitating valid webhooks. The HMAC digest signature, generated
    using SHA-1, will be stored in a X-Signature header.

    When you receive an event, you should compute a hash using your
    secret token, and ensure that the X-Signature sent by Amiqus
    matches that hash.

    If the HMAC signatures don't match, return a 403

    """

    def decorator(func: Callable) -> Callable:
        @wraps(func)
        def _wrapped_func(
            request: HttpRequest, *args: Any, **kwargs: Any
        ) -> HttpResponse:
            if TEST_MODE:
                logger.debug(
                    "Ignoring Amiqus callback verification (AMIQUS_TEST_MODE enabled)"
                )
                return func(request, *args, **kwargs)
            if not WEBHOOK_SECURITY_TOKEN:
                raise ImproperlyConfigured("Missing AMIQUS_WEBHOOK_SECURITY_TOKEN")
            if _match(WEBHOOK_SECURITY_TOKEN, request):
                return func(request, *args, **kwargs)
            else:
                # logging as a warning means it'll likely appear in logs,
                # but it's by design - if people are sending invalid requests
                # we need to know.
                logger.warning("Amiqus callback request verification failed.")
                return HttpResponseForbidden("Invalid X-Signature")

        return _wrapped_func

    return decorator



from __future__ import annotations

from dataclasses import dataclass


@dataclass
class AmiqusCountry:
    name: str
    alpha3: str
    region: str
    subregion: str
    supported_identity_report: bool


COUNTRY_DATA_LIST = [
    {
        "name": "Afghanistan",
        "alpha3": "AFG",
        "region": "Asia",
        "subregion": "Southern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Albania",
        "alpha3": "ALB",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Algeria",
        "alpha3": "DZA",
        "region": "Africa",
        "subregion": "Northern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "American Samoa",
        "alpha3": "ASM",
        "region": "Oceania",
        "subregion": "Polynesia",
        "supported_identity_report": False,
    },
    {
        "name": "Andorra",
        "alpha3": "AND",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Angola",
        "alpha3": "AGO",
        "region": "Africa",
        "subregion": "Middle Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Anguilla",
        "alpha3": "AIA",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": False,
    },
    {
        "name": "Antarctica",
        "alpha3": "ATA",
        "region": "",
        "subregion": "",
        "supported_identity_report": False,
    },
    {
        "name": "Antigua and Barbuda",
        "alpha3": "ATG",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Argentina",
        "alpha3": "ARG",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": True,
    },
    {
        "name": "Armenia",
        "alpha3": "ARM",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Aruba",
        "alpha3": "ABW",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Australia",
        "alpha3": "AUS",
        "region": "Oceania",
        "subregion": "Australia and New Zealand",
        "supported_identity_report": True,
    },
    {
        "name": "Austria",
        "alpha3": "AUT",
        "region": "Europe",
        "subregion": "Western Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Azerbaijan",
        "alpha3": "AZE",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Bahamas",
        "alpha3": "BHS",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": False,
    },
    {
        "name": "Bahrain",
        "alpha3": "BHR",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Bangladesh",
        "alpha3": "BGD",
        "region": "Asia",
        "subregion": "Southern Asia",
        "supported_identity_report": False,
    },
    {
        "name": "Barbados",
        "alpha3": "BRB",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Belarus",
        "alpha3": "BLR",
        "region": "Europe",
        "subregion": "Eastern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Belgium",
        "alpha3": "BEL",
        "region": "Europe",
        "subregion": "Western Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Belize",
        "alpha3": "BLZ",
        "region": "Americas",
        "subregion": "Central America",
        "supported_identity_report": True,
    },
    {
        "name": "Benin",
        "alpha3": "BEN",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Bermuda",
        "alpha3": "BMU",
        "region": "Americas",
        "subregion": "Northern America",
        "supported_identity_report": True,
    },
    {
        "name": "Bhutan",
        "alpha3": "BTN",
        "region": "Asia",
        "subregion": "Southern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Bolivia (Plurinational State of)",
        "alpha3": "BOL",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": False,
    },
    {
        "name": "Bonaire, Sint Eustatius and Saba",
        "alpha3": "BES",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": False,
    },
    {
        "name": "Bosnia and Herzegovina",
        "alpha3": "BIH",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Botswana",
        "alpha3": "BWA",
        "region": "Africa",
        "subregion": "Southern Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Bouvet Island",
        "alpha3": "BVT",
        "region": "",
        "subregion": "",
        "supported_identity_report": False,
    },
    {
        "name": "Brazil",
        "alpha3": "BRA",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": True,
    },
    {
        "name": "British Indian Ocean Territory",
        "alpha3": "IOT",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Brunei Darussalam",
        "alpha3": "BRN",
        "region": "Asia",
        "subregion": "South-Eastern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Bulgaria",
        "alpha3": "BGR",
        "region": "Europe",
        "subregion": "Eastern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Burkina Faso",
        "alpha3": "BFA",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Burundi",
        "alpha3": "BDI",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Cabo Verde",
        "alpha3": "CPV",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Cambodia",
        "alpha3": "KHM",
        "region": "Asia",
        "subregion": "South-Eastern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Cameroon",
        "alpha3": "CMR",
        "region": "Africa",
        "subregion": "Middle Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Canada",
        "alpha3": "CAN",
        "region": "Americas",
        "subregion": "Northern America",
        "supported_identity_report": True,
    },
    {
        "name": "Cayman Islands",
        "alpha3": "CYM",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Central African Republic",
        "alpha3": "CAF",
        "region": "Africa",
        "subregion": "Middle Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Chad",
        "alpha3": "TCD",
        "region": "Africa",
        "subregion": "Middle Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Chile",
        "alpha3": "CHL",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": True,
    },
    {
        "name": "China",
        "alpha3": "CHN",
        "region": "Asia",
        "subregion": "Eastern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Christmas Island",
        "alpha3": "CXR",
        "region": "Oceania",
        "subregion": "Australia and New Zealand",
        "supported_identity_report": False,
    },
    {
        "name": "Cocos (Keeling) Islands",
        "alpha3": "CCK",
        "region": "Oceania",
        "subregion": "Australia and New Zealand",
        "supported_identity_report": False,
    },
    {
        "name": "Colombia",
        "alpha3": "COL",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": True,
    },
    {
        "name": "Comoros",
        "alpha3": "COM",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Congo",
        "alpha3": "COG",
        "region": "Africa",
        "subregion": "Middle Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Congo (Democratic Republic of the)",
        "alpha3": "COD",
        "region": "Africa",
        "subregion": "Middle Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Cook Islands",
        "alpha3": "COK",
        "region": "Oceania",
        "subregion": "Polynesia",
        "supported_identity_report": True,
    },
    {
        "name": "Costa Rica",
        "alpha3": "CRI",
        "region": "Americas",
        "subregion": "Central America",
        "supported_identity_report": True,
    },
    {
        "name": "Croatia",
        "alpha3": "HRV",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Cuba",
        "alpha3": "CUB",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Curaçao",
        "alpha3": "CUW",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": False,
    },
    {
        "name": "Cyprus",
        "alpha3": "CYP",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Czech Republic",
        "alpha3": "CZE",
        "region": "Europe",
        "subregion": "Eastern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Côte d'Ivoire",
        "alpha3": "CIV",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Denmark",
        "alpha3": "DNK",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Djibouti",
        "alpha3": "DJI",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Dominica",
        "alpha3": "DMA",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Dominican Republic",
        "alpha3": "DOM",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Ecuador",
        "alpha3": "ECU",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": True,
    },
    {
        "name": "Egypt",
        "alpha3": "EGY",
        "region": "Africa",
        "subregion": "Northern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "El Salvador",
        "alpha3": "SLV",
        "region": "Americas",
        "subregion": "Central America",
        "supported_identity_report": True,
    },
    {
        "name": "Equatorial Guinea",
        "alpha3": "GNQ",
        "region": "Africa",
        "subregion": "Middle Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Eritrea",
        "alpha3": "ERI",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Estonia",
        "alpha3": "EST",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Ethiopia",
        "alpha3": "ETH",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Falkland Islands (Malvinas)",
        "alpha3": "FLK",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": False,
    },
    {
        "name": "Faroe Islands",
        "alpha3": "FRO",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Fiji",
        "alpha3": "FJI",
        "region": "Oceania",
        "subregion": "Melanesia",
        "supported_identity_report": True,
    },
    {
        "name": "Finland",
        "alpha3": "FIN",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "France",
        "alpha3": "FRA",
        "region": "Europe",
        "subregion": "Western Europe",
        "supported_identity_report": True,
    },
    {
        "name": "French Guiana",
        "alpha3": "GUF",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": False,
    },
    {
        "name": "French Polynesia",
        "alpha3": "PYF",
        "region": "Oceania",
        "subregion": "Polynesia",
        "supported_identity_report": False,
    },
    {
        "name": "French Southern Territories",
        "alpha3": "ATF",
        "region": "",
        "subregion": "",
        "supported_identity_report": False,
    },
    {
        "name": "Gabon",
        "alpha3": "GAB",
        "region": "Africa",
        "subregion": "Middle Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Gambia",
        "alpha3": "GMB",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Georgia",
        "alpha3": "GEO",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Germany",
        "alpha3": "DEU",
        "region": "Europe",
        "subregion": "Western Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Ghana",
        "alpha3": "GHA",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Gibraltar",
        "alpha3": "GIB",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Greece",
        "alpha3": "GRC",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Greenland",
        "alpha3": "GRL",
        "region": "Americas",
        "subregion": "Northern America",
        "supported_identity_report": False,
    },
    {
        "name": "Grenada",
        "alpha3": "GRD",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Guadeloupe",
        "alpha3": "GLP",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": False,
    },
    {
        "name": "Guam",
        "alpha3": "GUM",
        "region": "Oceania",
        "subregion": "Micronesia",
        "supported_identity_report": False,
    },
    {
        "name": "Guatemala",
        "alpha3": "GTM",
        "region": "Americas",
        "subregion": "Central America",
        "supported_identity_report": True,
    },
    {
        "name": "Guernsey",
        "alpha3": "GGY",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Guinea",
        "alpha3": "GIN",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Guinea-Bissau",
        "alpha3": "GNB",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Guyana",
        "alpha3": "GUY",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": False,
    },
    {
        "name": "Haiti",
        "alpha3": "HTI",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": False,
    },
    {
        "name": "Heard Island and McDonald Islands",
        "alpha3": "HMD",
        "region": "",
        "subregion": "",
        "supported_identity_report": False,
    },
    {
        "name": "Holy See",
        "alpha3": "VAT",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": False,
    },
    {
        "name": "Honduras",
        "alpha3": "HND",
        "region": "Americas",
        "subregion": "Central America",
        "supported_identity_report": True,
    },
    {
        "name": "Hong Kong",
        "alpha3": "HKG",
        "region": "Asia",
        "subregion": "Eastern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Hungary",
        "alpha3": "HUN",
        "region": "Europe",
        "subregion": "Eastern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Iceland",
        "alpha3": "ISL",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "India",
        "alpha3": "IND",
        "region": "Asia",
        "subregion": "Southern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Indonesia",
        "alpha3": "IDN",
        "region": "Asia",
        "subregion": "South-Eastern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Iran (Islamic Republic of)",
        "alpha3": "IRN",
        "region": "Asia",
        "subregion": "Southern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Iraq",
        "alpha3": "IRQ",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": False,
    },
    {
        "name": "Ireland",
        "alpha3": "IRL",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Isle of Man",
        "alpha3": "IMN",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Israel",
        "alpha3": "ISR",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Italy",
        "alpha3": "ITA",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Jamaica",
        "alpha3": "JAM",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Japan",
        "alpha3": "JPN",
        "region": "Asia",
        "subregion": "Eastern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Jersey",
        "alpha3": "JEY",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Jordan",
        "alpha3": "JOR",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": False,
    },
    {
        "name": "Kazakhstan",
        "alpha3": "KAZ",
        "region": "Asia",
        "subregion": "Central Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Kenya",
        "alpha3": "KEN",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Kiribati",
        "alpha3": "KIR",
        "region": "Oceania",
        "subregion": "Micronesia",
        "supported_identity_report": False,
    },
    {
        "name": "Korea (Democratic People's Republic of)",
        "alpha3": "PRK",
        "region": "Asia",
        "subregion": "Eastern Asia",
        "supported_identity_report": False,
    },
    {
        "name": "Korea (Republic of)",
        "alpha3": "KOR",
        "region": "Asia",
        "subregion": "Eastern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Kuwait",
        "alpha3": "KWT",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Kyrgyzstan",
        "alpha3": "KGZ",
        "region": "Asia",
        "subregion": "Central Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Lao People's Democratic Republic",
        "alpha3": "LAO",
        "region": "Asia",
        "subregion": "South-Eastern Asia",
        "supported_identity_report": False,
    },
    {
        "name": "Latvia",
        "alpha3": "LVA",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Lebanon",
        "alpha3": "LBN",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Lesotho",
        "alpha3": "LSO",
        "region": "Africa",
        "subregion": "Southern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Liberia",
        "alpha3": "LBR",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Libya",
        "alpha3": "LBY",
        "region": "Africa",
        "subregion": "Northern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Liechtenstein",
        "alpha3": "LIE",
        "region": "Europe",
        "subregion": "Western Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Lithuania",
        "alpha3": "LTU",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Luxembourg",
        "alpha3": "LUX",
        "region": "Europe",
        "subregion": "Western Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Macao",
        "alpha3": "MAC",
        "region": "Asia",
        "subregion": "Eastern Asia",
        "supported_identity_report": False,
    },
    {
        "name": "Madagascar",
        "alpha3": "MDG",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Malawi",
        "alpha3": "MWI",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Malaysia",
        "alpha3": "MYS",
        "region": "Asia",
        "subregion": "South-Eastern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Maldives",
        "alpha3": "MDV",
        "region": "Asia",
        "subregion": "Southern Asia",
        "supported_identity_report": False,
    },
    {
        "name": "Mali",
        "alpha3": "MLI",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Malta",
        "alpha3": "MLT",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Marshall Islands",
        "alpha3": "MHL",
        "region": "Oceania",
        "subregion": "Micronesia",
        "supported_identity_report": False,
    },
    {
        "name": "Martinique",
        "alpha3": "MTQ",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": False,
    },
    {
        "name": "Mauritania",
        "alpha3": "MRT",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Mauritius",
        "alpha3": "MUS",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Mayotte",
        "alpha3": "MYT",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Mexico",
        "alpha3": "MEX",
        "region": "Americas",
        "subregion": "Central America",
        "supported_identity_report": True,
    },
    {
        "name": "Micronesia (Federated States of)",
        "alpha3": "FSM",
        "region": "Oceania",
        "subregion": "Micronesia",
        "supported_identity_report": False,
    },
    {
        "name": "Moldova (Republic of)",
        "alpha3": "MDA",
        "region": "Europe",
        "subregion": "Eastern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Monaco",
        "alpha3": "MCO",
        "region": "Europe",
        "subregion": "Western Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Mongolia",
        "alpha3": "MNG",
        "region": "Asia",
        "subregion": "Eastern Asia",
        "supported_identity_report": False,
    },
    {
        "name": "Montenegro",
        "alpha3": "MNE",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Montserrat",
        "alpha3": "MSR",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Morocco",
        "alpha3": "MAR",
        "region": "Africa",
        "subregion": "Northern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Mozambique",
        "alpha3": "MOZ",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Myanmar",
        "alpha3": "MMR",
        "region": "Asia",
        "subregion": "South-Eastern Asia",
        "supported_identity_report": False,
    },
    {
        "name": "Namibia",
        "alpha3": "NAM",
        "region": "Africa",
        "subregion": "Southern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Nauru",
        "alpha3": "NRU",
        "region": "Oceania",
        "subregion": "Micronesia",
        "supported_identity_report": False,
    },
    {
        "name": "Nepal",
        "alpha3": "NPL",
        "region": "Asia",
        "subregion": "Southern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Netherlands",
        "alpha3": "NLD",
        "region": "Europe",
        "subregion": "Western Europe",
        "supported_identity_report": True,
    },
    {
        "name": "New Caledonia",
        "alpha3": "NCL",
        "region": "Oceania",
        "subregion": "Melanesia",
        "supported_identity_report": False,
    },
    {
        "name": "New Zealand",
        "alpha3": "NZL",
        "region": "Oceania",
        "subregion": "Australia and New Zealand",
        "supported_identity_report": True,
    },
    {
        "name": "Nicaragua",
        "alpha3": "NIC",
        "region": "Americas",
        "subregion": "Central America",
        "supported_identity_report": True,
    },
    {
        "name": "Niger",
        "alpha3": "NER",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Nigeria",
        "alpha3": "NGA",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Niue",
        "alpha3": "NIU",
        "region": "Oceania",
        "subregion": "Polynesia",
        "supported_identity_report": False,
    },
    {
        "name": "Norfolk Island",
        "alpha3": "NFK",
        "region": "Oceania",
        "subregion": "Australia and New Zealand",
        "supported_identity_report": True,
    },
    {
        "name": "North Macedonia",
        "alpha3": "MKD",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": False,
    },
    {
        "name": "Northern Mariana Islands",
        "alpha3": "MNP",
        "region": "Oceania",
        "subregion": "Micronesia",
        "supported_identity_report": False,
    },
    {
        "name": "Norway",
        "alpha3": "NOR",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Oman",
        "alpha3": "OMN",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Pakistan",
        "alpha3": "PAK",
        "region": "Asia",
        "subregion": "Southern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Palau",
        "alpha3": "PLW",
        "region": "Oceania",
        "subregion": "Micronesia",
        "supported_identity_report": False,
    },
    {
        "name": "Palestine, State of",
        "alpha3": "PSE",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Panama",
        "alpha3": "PAN",
        "region": "Americas",
        "subregion": "Central America",
        "supported_identity_report": True,
    },
    {
        "name": "Papua New Guinea",
        "alpha3": "PNG",
        "region": "Oceania",
        "subregion": "Melanesia",
        "supported_identity_report": False,
    },
    {
        "name": "Paraguay",
        "alpha3": "PRY",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": True,
    },
    {
        "name": "Peru",
        "alpha3": "PER",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": True,
    },
    {
        "name": "Philippines",
        "alpha3": "PHL",
        "region": "Asia",
        "subregion": "South-Eastern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Pitcairn",
        "alpha3": "PCN",
        "region": "Oceania",
        "subregion": "Polynesia",
        "supported_identity_report": False,
    },
    {
        "name": "Poland",
        "alpha3": "POL",
        "region": "Europe",
        "subregion": "Eastern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Portugal",
        "alpha3": "PRT",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Puerto Rico",
        "alpha3": "PRI",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Qatar",
        "alpha3": "QAT",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Romania",
        "alpha3": "ROU",
        "region": "Europe",
        "subregion": "Eastern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Russian Federation",
        "alpha3": "RUS",
        "region": "Europe",
        "subregion": "Eastern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Rwanda",
        "alpha3": "RWA",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Réunion",
        "alpha3": "REU",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Saint Barthélemy",
        "alpha3": "BLM",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": False,
    },
    {
        "name": "Saint Helena, Ascension and Tristan da Cunha",
        "alpha3": "SHN",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Saint Kitts and Nevis",
        "alpha3": "KNA",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Saint Lucia",
        "alpha3": "LCA",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Saint Martin (French part)",
        "alpha3": "MAF",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": False,
    },
    {
        "name": "Saint Pierre and Miquelon",
        "alpha3": "SPM",
        "region": "Americas",
        "subregion": "Northern America",
        "supported_identity_report": False,
    },
    {
        "name": "Saint Vincent and the Grenadines",
        "alpha3": "VCT",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Samoa",
        "alpha3": "WSM",
        "region": "Oceania",
        "subregion": "Polynesia",
        "supported_identity_report": True,
    },
    {
        "name": "San Marino",
        "alpha3": "SMR",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": False,
    },
    {
        "name": "Sao Tome and Principe",
        "alpha3": "STP",
        "region": "Africa",
        "subregion": "Middle Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Saudi Arabia",
        "alpha3": "SAU",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Senegal",
        "alpha3": "SEN",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Serbia",
        "alpha3": "SRB",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Seychelles",
        "alpha3": "SYC",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Sierra Leone",
        "alpha3": "SLE",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Singapore",
        "alpha3": "SGP",
        "region": "Asia",
        "subregion": "South-Eastern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Sint Maarten (Dutch part)",
        "alpha3": "SXM",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": False,
    },
    {
        "name": "Slovakia",
        "alpha3": "SVK",
        "region": "Europe",
        "subregion": "Eastern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Slovenia",
        "alpha3": "SVN",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Solomon Islands",
        "alpha3": "SLB",
        "region": "Oceania",
        "subregion": "Melanesia",
        "supported_identity_report": False,
    },
    {
        "name": "Somalia",
        "alpha3": "SOM",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "South Africa",
        "alpha3": "ZAF",
        "region": "Africa",
        "subregion": "Southern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "South Georgia and the South Sandwich Islands",
        "alpha3": "SGS",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": False,
    },
    {
        "name": "South Sudan",
        "alpha3": "SSD",
        "region": "Africa",
        "subregion": "Northern Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Spain",
        "alpha3": "ESP",
        "region": "Europe",
        "subregion": "Southern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Sri Lanka",
        "alpha3": "LKA",
        "region": "Asia",
        "subregion": "Southern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Sudan",
        "alpha3": "SDN",
        "region": "Africa",
        "subregion": "Northern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Suriname",
        "alpha3": "SUR",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": False,
    },
    {
        "name": "Svalbard and Jan Mayen",
        "alpha3": "SJM",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": False,
    },
    {
        "name": "Sweden",
        "alpha3": "SWE",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Switzerland",
        "alpha3": "CHE",
        "region": "Europe",
        "subregion": "Western Europe",
        "supported_identity_report": True,
    },
    {
        "name": "Syrian Arab Republic",
        "alpha3": "SYR",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Taiwan, Province of China",
        "alpha3": "TWN",
        "region": "Asia",
        "subregion": "Eastern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Tajikistan",
        "alpha3": "TJK",
        "region": "Asia",
        "subregion": "Central Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Tanzania, United Republic of",
        "alpha3": "TZA",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Thailand",
        "alpha3": "THA",
        "region": "Asia",
        "subregion": "South-Eastern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Timor-Leste",
        "alpha3": "TLS",
        "region": "Asia",
        "subregion": "South-Eastern Asia",
        "supported_identity_report": False,
    },
    {
        "name": "Togo",
        "alpha3": "TGO",
        "region": "Africa",
        "subregion": "Western Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Tokelau",
        "alpha3": "TKL",
        "region": "Oceania",
        "subregion": "Polynesia",
        "supported_identity_report": False,
    },
    {
        "name": "Tonga",
        "alpha3": "TON",
        "region": "Oceania",
        "subregion": "Polynesia",
        "supported_identity_report": False,
    },
    {
        "name": "Trinidad and Tobago",
        "alpha3": "TTO",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Tunisia",
        "alpha3": "TUN",
        "region": "Africa",
        "subregion": "Northern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Turkey",
        "alpha3": "TUR",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Turkmenistan",
        "alpha3": "TKM",
        "region": "Asia",
        "subregion": "Central Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Turks and Caicos Islands",
        "alpha3": "TCA",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Tuvalu",
        "alpha3": "TUV",
        "region": "Oceania",
        "subregion": "Polynesia",
        "supported_identity_report": True,
    },
    {
        "name": "Uganda",
        "alpha3": "UGA",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Ukraine",
        "alpha3": "UKR",
        "region": "Europe",
        "subregion": "Eastern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "United Arab Emirates",
        "alpha3": "ARE",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": True,
    },
    {
        "name": "United Kingdom of Great Britain and Northern Ireland",
        "alpha3": "GBR",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
    {
        "name": "United States Minor Outlying Islands",
        "alpha3": "UMI",
        "region": "Americas",
        "subregion": "Northern America",
        "supported_identity_report": False,
    },
    {
        "name": "United States of America",
        "alpha3": "USA",
        "region": "Americas",
        "subregion": "Northern America",
        "supported_identity_report": True,
    },
    {
        "name": "Uruguay",
        "alpha3": "URY",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": False,
    },
    {
        "name": "Uzbekistan",
        "alpha3": "UZB",
        "region": "Asia",
        "subregion": "Central Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Vanuatu",
        "alpha3": "VUT",
        "region": "Oceania",
        "subregion": "Melanesia",
        "supported_identity_report": True,
    },
    {
        "name": "Venezuela (Bolivarian Republic of)",
        "alpha3": "VEN",
        "region": "Americas",
        "subregion": "South America",
        "supported_identity_report": True,
    },
    {
        "name": "Viet Nam",
        "alpha3": "VNM",
        "region": "Asia",
        "subregion": "South-Eastern Asia",
        "supported_identity_report": True,
    },
    {
        "name": "Virgin Islands (British)",
        "alpha3": "VGB",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": True,
    },
    {
        "name": "Virgin Islands (U.S.)",
        "alpha3": "VIR",
        "region": "Americas",
        "subregion": "Caribbean",
        "supported_identity_report": False,
    },
    {
        "name": "Wallis and Futuna",
        "alpha3": "WLF",
        "region": "Oceania",
        "subregion": "Polynesia",
        "supported_identity_report": False,
    },
    {
        "name": "Western Sahara",
        "alpha3": "ESH",
        "region": "Africa",
        "subregion": "Northern Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Yemen",
        "alpha3": "YEM",
        "region": "Asia",
        "subregion": "Western Asia",
        "supported_identity_report": False,
    },
    {
        "name": "Zambia",
        "alpha3": "ZMB",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "Zimbabwe",
        "alpha3": "ZWE",
        "region": "Africa",
        "subregion": "Eastern Africa",
        "supported_identity_report": True,
    },
    {
        "name": "eSwatini",
        "alpha3": "SWZ",
        "region": "Africa",
        "subregion": "Southern Africa",
        "supported_identity_report": False,
    },
    {
        "name": "Åland Islands",
        "alpha3": "ALA",
        "region": "Europe",
        "subregion": "Northern Europe",
        "supported_identity_report": True,
    },
]


# raw data converted to a dict for faster lookup
COUNTRY_DATA_DICT = {c["alpha3"]: c for c in COUNTRY_DATA_LIST}


def get_country(alpha3: str) -> AmiqusCountry | None:
    if country_data := COUNTRY_DATA_DICT.get(alpha3):
        return AmiqusCountry(**country_data)  # type: ignore
    return None



from os import getenv

from django.conf import settings


def _setting(key, default):
    return getenv(key, default) or getattr(settings, key, default)


# API key from evnironment by default
API_KEY = _setting("AMIQUS_ACCESS_TOKEN", None)

# Webhook token - see https://documentation.onfido.com/#webhooks
WEBHOOK_SECURITY_TOKEN = _setting("AMIQUS_WEBHOOK_SECURITY_TOKEN", None)
# token must be a bytestring for HMAC function to work
WEBHOOK_SECURITY_TOKEN = (
    str.encode(WEBHOOK_SECURITY_TOKEN) if WEBHOOK_SECURITY_TOKEN else None
)

# Set to False to turn off event logging
LOG_EVENTS = _setting("AMIQUS_LOG_EVENTS", True)

# Set to True to bypass request verification (NOT RECOMMENDED)
TEST_MODE = _setting("AMIQUS_TEST_MODE", True)


def DEFAULT_CHECK_SCRUBBER(raw):
    """Remove specified properties."""
    return {k: v for k, v in raw.items() if k not in ("email")}


def DEFAULT_CLIENT_SCRUBBER(raw):
    """Remove _all_ data except the given properties."""
    return {k: v for k, v in raw.items() if k in ("id", "status", "created_at")}


# functions used to scrub sensitive data from checks
scrub_check_data = (
    getattr(settings, "AMIQUS_CHECK_SCRUBBER", None) or DEFAULT_CHECK_SCRUBBER
)

scrub_client_data = (
    getattr(settings, "AMIQUS_CLIENT_SCRUBBER", None) or DEFAULT_CLIENT_SCRUBBER
)

DEFAULT_REQUESTS_TIMEOUT = _setting("DEFAULT_REQUESTS_TIMEOUT", 30)
